from messagebus import TokenCreator, TokenPriority

SERVICE_LOCATOR_GROUP = "3"
TK_SERVICE_LOCATOR_REGISTER_SERVICE = TokenCreator.create_token(TokenPriority.high, SERVICE_LOCATOR_GROUP, "1")
TK_SERVICE_LOCATOR_LOOKUP_SERVICE = TokenCreator.create_token(TokenPriority.high, SERVICE_LOCATOR_GROUP, "2")
TK_SERVICE_LOCATOR_KEEP_ALIVE = TokenCreator.create_token(TokenPriority.high, SERVICE_LOCATOR_GROUP, "3")
