from keepalive_tests import *  # noqa: We want all tests to run
from locate_tests import *  # noqa: We want all tests to run
from messagehandlerbuilder import StoreStrategy
from register_tests import *  # noqa: We want all tests to run


@pytest.fixture
def store_strategy():
    return StoreStrategy.IN_MEMORY
